## mainWindow.py - Contains the main UI for system-config-samba
## Copyright (C) 2002, 2003 Red Hat, Inc.
## Copyright (C) 2002, 2003 Brent Fox <bfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


import sambaBackend
import sambaParser
import sambaToken
import gtk
import gobject
import string
import os
import gtk.glade
import basicPreferencesWin
import sambaUserWin
import sambaUserData
import shareWindow

##
## I18N
##
from rhpl.translate import _, N_
import rhpl.translate as translate
domain = 'system-config-samba'
translate.textdomain (domain)
gtk.glade.bindtextdomain(domain)

##
## Icon for windows
##

iconPixbuf = None      
try:
    iconPixbuf = gtk.gdk.pixbuf_new_from_file("/usr/share/system-config-samba/pixmaps/system-config-samba.png")
except:
    pass

class MainWindow:
    def __init__(self, debug_flag=None):
        self.debug_flag = debug_flag

        if os.access("system-config-samba.glade", os.F_OK):
            self.xml = gtk.glade.XML ("system-config-samba.glade", domain="system-config-samba")
        else:
            self.xml = gtk.glade.XML ("/usr/share/system-config-samba/system-config-samba.glade", domain="system-config-samba")

            
#        [0 dir, 1 hosts, 2 read/write text, 3 read/write val, 4 sambaDataObject]
        self.share_store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING,
                                         gobject.TYPE_STRING, gobject.TYPE_STRING)

        self.main_window = gtk.Window()
        self.main_window.set_title(_("Samba Server Configuration"))
        self.main_window.connect("delete-event", self.destroy)
        self.main_window.set_position(gtk.WIN_POS_CENTER)
        self.main_window.set_icon(iconPixbuf)
        self.nameTag = _("Samba")
        self.commentTag = _("Create, modify, and delete samba shares")

        #Initialize Global Samba data store
        self.samba_data = sambaParser.SambaParser(self)
        self.processSambaData(self.samba_data)

        #Initialize Samba Backend
        self.samba_backend = sambaBackend.SambaBackend()

        #Initialize Samba User Backend
        self.samba_user_data = sambaUserData.SambaUserData(self)

        #Initialize Samba User Window
        self.samba_user_win = sambaUserWin.SambaUserWin(self, self.xml, self.samba_user_data, self.main_window)

        self.basic_preferences_win = basicPreferencesWin.BasicPreferencesWin(self, self.xml, self.samba_data, self.samba_backend, self.main_window)
        self.share_win = shareWindow.ShareWindow(self, self.xml, self.samba_data, self.samba_user_data, self.samba_backend, self.main_window)
        
        self.toplevel_vbox = gtk.VBox(gtk.FALSE)
        self.menu_bar = gtk.MenuBar()

        self.share_view = gtk.TreeView(self.share_store)
        self.share_view.set_rules_hint(gtk.TRUE)
        self.share_view.columns_autosize()
        self.share_view_sw = gtk.ScrolledWindow()
        self.share_view_sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.share_view_sw.set_shadow_type(gtk.SHADOW_IN)
        self.share_view_sw.add(self.share_view)

        col = gtk.TreeViewColumn(_("Directory"), gtk.CellRendererText(), text=0)
#        col.set_spacing(235)
        self.share_view.append_column(col)
        col = gtk.TreeViewColumn(_("Share name"), gtk.CellRendererText(), text=1)
        self.share_view.append_column(col)
        col = gtk.TreeViewColumn(_("Permissions"), gtk.CellRendererText(), text=2)
        self.share_view.append_column(col)
        col = gtk.TreeViewColumn(_("Description"), gtk.CellRendererText(), text=3)   
        col.set_spacing(235)
        self.share_view.append_column(col)

        accel_group = gtk.AccelGroup()
        item_fac = gtk.ItemFactory(gtk.MenuBar, "<main>", accel_group)

        self.main_window.add_accel_group(accel_group)
        item_fac.create_items([
            (_('/_File'),                   None,    None, 0, '<Branch>'),
            (_('/_File/_Add Share'),  None,    self.onNewButtonClicked, 6, '<StockItem>', gtk.STOCK_ADD),
            (_('/_File/_Properties'),       None,   self.onPropertiesButtonClicked, 6, '<StockItem>', gtk.STOCK_PROPERTIES),
            (_('/_File/_Delete'),           None,    self.onDeleteButtonClicked, 6, '<StockItem>', gtk.STOCK_DELETE),
            (_('/_File/_Quit'),             None,    self.destroy, 6, '<StockItem>', gtk.STOCK_QUIT),
            (_('/_Preferences'),                     None,    None, 0, '<Branch>'),
            (_('/_Preferences/_Server Settings...'),  None,    self.onBasicPreferencesClicked, 1, '<StockItem>', gtk.STOCK_PREFERENCES),
#            (_('/_Preferences/_Advanced Preferences...'),  None,   self.onAdvancedPreferencesClicked, 2, ''),
            (_('/_Preferences/Samba _Users...'),  None,    self.onModifyUsersClicked, 3, ''),
            (_('/_Help'),                     None,    None, 0, '<Branch>'),
            (_('/_Help/_Contents'),             None,    self.onHelpClicked, 1, '<StockItem>', gtk.STOCK_HELP),
            (_('/_Help/_About'),              None,    self.onAboutClicked, 2, ''),
            ])
        self.menu_bar = item_fac.get_widget('<main>')

        self.actionMenu = self.menu_bar.get_children()[0].get_submenu()        
        self.propertiesMenu = self.actionMenu.get_children()[1]
        self.deleteMenu = self.actionMenu.get_children()[2]

        self.toolbar = gtk.Toolbar()

        button = self.toolbar.insert_stock('gtk-add', _("Add a samba share"), None, self.onNewButtonClicked, None, 0)
        image, label = button.get_children()[0].get_children()
        label.set_text(_("_Add"))
        label.set_use_underline(gtk.TRUE)
        
        self.properties_button = self.toolbar.insert_stock('gtk-properties', _("Edit the properties of the selected directory"), None, self.onPropertiesButtonClicked, None, 1)
        image, label = self.properties_button.get_children()[0].get_children()
        label.set_text(_("P_roperties"))
        label.set_use_underline(gtk.TRUE)

        self.delete_button = self.toolbar.insert_stock('gtk-delete', _("Delete the selected directory"), None, self.onDeleteButtonClicked, None, 2)
        image, label = self.delete_button.get_children()[0].get_children()
        label.set_text(_("_Delete"))
        label.set_use_underline(gtk.TRUE)
        
        self.toolbar.insert_stock('gtk-help', _("View help"), None, self.onHelpClicked, None, 3)
        
        self.properties_button.set_sensitive(gtk.FALSE)
        self.delete_button.set_sensitive(gtk.FALSE)
        self.propertiesMenu.set_sensitive(gtk.FALSE)
        self.deleteMenu.set_sensitive(gtk.FALSE)

        #-------------------Packing--------------------#
        self.toplevel_vbox.pack_start(self.menu_bar, gtk.FALSE)        
        self.toplevel_vbox.pack_start(self.toolbar, gtk.FALSE)
        self.toplevel_vbox.pack_start(self.share_view_sw, gtk.TRUE)
        self.main_window.add(self.toplevel_vbox)
        self.main_window.set_default_size(600, 400)
        self.main_window.set_size_request(600, 200)

        self.selected_row = -1
        self.changed = gtk.FALSE
        
        self.create_handler = None
        self.modify_handler = None

        self.main_window.show_all()

#        self.share_view.connect("cursor-changed", self.onShareListSelectRow)
        self.share_view.get_selection().connect("changed", self.onShareListSelectRow)
        self.share_view.get_selection().unselect_all()
        
        if gtk.__dict__.has_key ("main"):
            gtk.main ()
        else:
            gtk.mainloop()

    def processSambaData(self, samba_data):
        list = samba_data.getList()
        header_list = samba_data.getShareHeaders()

        for header in header_list:
            iter = self.share_store.append()
            self.share_store.set_value(iter, 2, (_("Read Only")))
 
            for line in list:
                if line.type == sambaToken.SambaToken.SAMBA_TOKEN_SECTION_HEADER and string.strip(line.getData()) == header:
                    sharename = string.strip (line.getData(),"\n[]")
                    self.share_store.set_value(iter, 1, sharename)

                if line.type == sambaToken.SambaToken.SAMBA_TOKEN_KEYVAL and line.parent == header:
                    self.share_store.set_value(iter, 4, line.parent)
                    
                    if line.keyname == "path":
                        self.share_store.set_value(iter, 0, line.keyval)
                    elif line.keyname == "read only":

                        if string.lower(line.keyval) == "no":
                            self.share_store.set_value(iter, 2, (_("Read/Write")))
                        else:
                            self.share_store.set_value(iter, 2, (_("Read Only")))

                    elif line.keyname == "writeable" or line.keyname == "writable":
                        if string.lower(line.keyval) == "no":
                            self.share_store.set_value(iter, 2, (_("Read Only")))
                        else:
                            self.share_store.set_value(iter, 2, (_("Read/Write")))

                    elif line.keyname == "comment":
                        self.share_store.set_value(iter, 3, line.keyval)
                    
        pass

    def changeSectionHeader(self, oldname, newname):
        self.samba_file_list = self.samba_data.getList()

        found = gtk.FALSE
        last_global_line = 0

        for line in self.samba_file_list:

            if line.type == sambaToken.SambaToken.SAMBA_TOKEN_SECTION_HEADER:
                sectionname = string.strip (line.getData(),"\n[]")
                if sectionname == oldname:
                    line.value = "[" + newname + "]"
                    found = gtk.TRUE
                    break
                last_global_line = self.samba_file_list.index(line)
                
        if found == gtk.FALSE:
            line = "[" + newname + "]"
            token = self.samba_data.createToken(line)
            self.samba_file_list.insert(last_global_line, token)

    def changeValue(self, keyname, value, parent):
        self.samba_file_list = self.samba_data.getList()
        
        if value == None or string.strip(value) == "":
            return

        found = gtk.FALSE
        last_global_line = 0

        for line in self.samba_file_list:
            parent_tag = line.getParent()

            if line.type == sambaToken.SambaToken.SAMBA_TOKEN_KEYVAL and parent_tag == parent:
                if line.keyname == keyname:
                    line.keyval = value
                    found = gtk.TRUE
                    break
                last_global_line = self.samba_file_list.index(line)
                
        if found == gtk.FALSE:
            line = keyname + "=" + value
            token = self.samba_data.createToken(line, parent)
            self.samba_file_list.insert(last_global_line, token)
   

    #--------Event handlers for main_window-----#
    def onShareListSelectRow(self, *args):
        store, iter = self.share_view.get_selection().get_selected()
        if iter:
            self.properties_button.set_sensitive(gtk.TRUE)
            self.delete_button.set_sensitive(gtk.TRUE)
            self.propertiesMenu.set_sensitive(gtk.TRUE)
            self.deleteMenu.set_sensitive(gtk.TRUE)
        else:
            self.properties_button.set_sensitive(gtk.FALSE)
            self.delete_button.set_sensitive(gtk.FALSE)
            self.propertiesMenu.set_sensitive(gtk.FALSE)
            self.deleteMenu.set_sensitive(gtk.FALSE)

    def onNewButtonClicked(self, *args):
        self.changed = gtk.TRUE
        self.share_win.showNewWindow()

    def onPropertiesButtonClicked(self, *args):
        self.changed = gtk.TRUE
        store, iter = self.share_view.get_selection().get_selected()
        if iter != None:
            parent = self.share_store.get_value(iter, 4)
            self.share_win.showEditWindow(iter, parent)
        
    def onDeleteButtonClicked(self, *args):
        self.changed = gtk.TRUE

        try:
            store, iter = self.share_view.get_selection().get_selected()
            parent = self.share_store.get_value(iter, 4)
            self.share_store.remove(iter)

            tmp_list =  self.samba_data.getList()[:]

            for line in tmp_list:

                if line.type == sambaToken.SambaToken.SAMBA_TOKEN_SECTION_HEADER and line.value == parent:
                    self.samba_data.samba_file_list.remove(line)

                elif line.parent == parent:
                    self.samba_data.samba_file_list.remove(line)

            self.properties_button.set_sensitive(gtk.FALSE)
            self.delete_button.set_sensitive(gtk.FALSE)
            self.propertiesMenu.set_sensitive(gtk.FALSE)
            self.deleteMenu.set_sensitive(gtk.FALSE)

            #Let's go ahead and restart the service.
            self.samba_data.writeFile()
            self.samba_backend.restartSamba()

        except:
            pass

    def onApplyButtonClicked(self, *args):
        self.changed = gtk.TRUE

        self.samba_data.writeFile()
        self.properties_button.set_sensitive(gtk.FALSE)
        self.delete_button.set_sensitive(gtk.FALSE)
        self.propertiesMenu.set_sensitive(gtk.FALSE)
        self.deleteMenu.set_sensitive(gtk.FALSE)
        self.share_view.get_selection().unselect_all()

        status = self.samba_backend.isSmbRunning()
        
        result = None
        if not status:
            #smb is not running, so ask the user if they want to start it
            dlg = gtk.MessageDialog(self.main_window, 0, gtk.MESSAGE_WARNING,
                                    gtk.BUTTONS_YES_NO, (_("The Samba service is not "
                                                           "currently running.  Do you "
                                                           "wish to start it?")))
            dlg.set_position(gtk.WIN_POS_CENTER)
            dlg.set_icon(iconPixbuf)
            dlg.set_modal(gtk.TRUE)
            dlg.set_transient_for(self.main_window)
            result = dlg.run()
            dlg.destroy()
                                                           
            if result == gtk.RESPONSE_YES:
                #Start smb if the user says 'yes'
                if not self.debug_flag:
                    self.samba_backend.startSamba()
                else:
                    print "cannot start the service in debug mode"

        else:
            #smb is already running, so restart to allow changes to take effect
            if not self.debug_flag:
                self.samba_backend.restartSamba()
            else:
                print "cannot start the service in debug mode"

    def onBasicPreferencesClicked(self, *args):
        self.basic_preferences_win.showWindow()

    def onAdvancedPreferencesClicked(self, *args):
        advanced_preferences_win = self.xml.get_widget("advanced_preferences_win")
        advanced_preferences_win.show_all()

    def onRefreshButtonClicked(self, *args):
        pass

    def onModifyUsersClicked(self, *args):
        self.samba_user_win.showWindow()

    def destroy(self, *args):
#        #Maybe we don't need to restart the service now, so let's comment this line out
#        self.onApplyButtonClicked()
        if gtk.__dict__.has_key ("main_quit"):
            gtk.main_quit ()
        else:
            gtk.mainquit()

    def onAboutClicked(self, action, widget):
        dlg = gtk.MessageDialog (None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK,
                                 _("Samba Server Configuration Tool %s\n Copyright (c) 2002 Red Hat, Inc.\n Copyright (c) 2002 Brent Fox <bfox@redhat.com>\n Copyright (c) 2002 Tammy Fox <tfox@redhat.com>\n A graphical interface for configuring SMB shares") % ("1.2.21"))
        dlg.set_title(_("About"))
        dlg.set_default_size(100, 100)
        dlg.set_position (gtk.WIN_POS_CENTER)
        dlg.set_border_width(2)
        dlg.set_modal(gtk.TRUE)
        dlg.set_transient_for(self.main_window)
        dlg.set_icon(iconPixbuf)
        rc = dlg.run()
        dlg.destroy()

    def onHelpClicked(self, *args):
        help_page = "file:///usr/share/doc/system-config-samba-" + "1.2.21" + "/index.html"
	path = "/usr/bin/htmlview"
        
        if path == None:
            dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                    (_("Help is not available.")))
            dlg.set_position(gtk.WIN_POS_CENTER)
            dlg.run()
            dlg.destroy()
            return
		
        pid = os.fork()
        if not pid:
            os.execv(path, [path, help_page])
