## shareWindow.py - the UI code for creating samba shares
## Copyright (C) 2002, 2003 Red Hat, Inc.
## Copyright (C) 2002, 2003 Brent Fox <bfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
import string
import mainWindow
import sambaToken
import sambaUserData
import sambaParser
import os
import gobject

##
## I18N
##
from rhpl.translate import _, N_
import rhpl.translate as translate
domain = 'system-config-samba'
translate.textdomain (domain)
gtk.glade.bindtextdomain(domain)

class ShareWindow:
    def __init__(self, parent, xml, samba_data, samba_user_data, samba_backend, main_window):
        self.ParentClass = parent
        self.samba_data = samba_data
        self.samba_user_data = samba_user_data
        self.samba_backend = samba_backend
        self.samba_file_list = samba_data.getList()
        
        self.share_window = xml.get_widget("share_win")
        self.share_window.set_modal(gtk.TRUE)
        self.share_window.set_transient_for(main_window)
        self.share_window.connect("delete-event", self.onCancelButtonClicked)
        self.share_window.set_position(gtk.WIN_POS_CENTER)
        self.share_window.set_icon(mainWindow.iconPixbuf)
        self.share_notebook = xml.get_widget("share_notebook")
        self.dir_entry = xml.get_widget("share_dir_entry")
        self.description_entry = xml.get_widget("description_entry")
        self.sharename_entry = xml.get_widget("sharename_entry")
        self.read_radio = xml.get_widget("share_read_radio")
        self.write_radio = xml.get_widget("share_write_radio")

        self.user_access_radio = xml.get_widget("user_access_radio")
        self.guest_access_radio = xml.get_widget("guest_access_radio")
        self.user_access_radio.connect("toggled", self.userRadioToggled)
        
        
        self.ok_button = xml.get_widget("share_ok_button")

        xml.signal_connect("on_share_cancel_button_clicked", self.onCancelButtonClicked)
        xml.signal_connect("on_share_browse_button_clicked", self.onBrowseButtonClicked)
        xml.signal_connect("on_share_dir_entry_changed", self.onDirEntryChanged)
        xml.signal_connect("on_sharename_entry_changed", self.onShareNameEntryChanged)

        self.valid_users_treeview = xml.get_widget("valid_users_treeview")
        self.browsable_checkbutton = xml.get_widget("browsable_checkbutton")
        self.create_mode_label = xml.get_widget("create_mode_label")
        self.dir_mode_label = xml.get_widget("dir_mode_label")
        self.create_mode_button = xml.get_widget("create_mode_button")
        self.dir_mode_button = xml.get_widget("dir_mode_button")        

        self.valid_users_store = gtk.ListStore(gobject.TYPE_BOOLEAN, gobject.TYPE_STRING)
        self.valid_users_treeview.set_model(self.valid_users_store)

        self.checkbox = gtk.CellRendererToggle()
        col = gtk.TreeViewColumn('', self.checkbox, active = 0)
        col.set_fixed_width(20)
        col.set_clickable(gtk.TRUE)
        self.checkbox.connect("toggled", self.userToggled)
        self.valid_users_treeview.append_column(col)

        col = gtk.TreeViewColumn("", gtk.CellRendererText(), text=1)
        self.valid_users_treeview.append_column(col)

    def populateUserStore(self):
        userList = self.samba_user_data.getPasswdFile()

        if userList == None:
            return

        userList.sort()
        for line in userList:
            iter = self.valid_users_store.append()
            tokens = string.split(line, ':')
            self.valid_users_store.set_value(iter, 0, gtk.FALSE)
            self.valid_users_store.set_value(iter, 1, tokens[0])

    def populateUserStoreOnEdit(self, currentUserList, invalidUsers):
        userList = self.samba_user_data.getPasswdFile()

        if userList == None:
            return

        userList.sort()
        for line in userList:
            iter = self.valid_users_store.append()
            tokens = string.split(line, ':')

            if self.guest_access_radio.get_active() == gtk.TRUE:
                self.valid_users_store.set_value(iter, 0, gtk.FALSE)
                self.valid_users_store.set_value(iter, 1, tokens[0])
                continue
                
            if invalidUsers == "%S":
                #Make all users unselected
                self.valid_users_store.set_value(iter, 0, gtk.FALSE)
                self.valid_users_store.set_value(iter, 1, tokens[0])
            else:
                #invalidUsers is not "%S"
                if currentUserList == ['None'] or currentUserList == []:
                    #If no users are specified, assume all are allowed
                    self.valid_users_store.set_value(iter, 0, gtk.TRUE)
                    self.valid_users_store.set_value(iter, 1, tokens[0])
                else:
                    #Let's see which users are allowed
                    if tokens[0] in currentUserList:
                        self.valid_users_store.set_value(iter, 0, gtk.TRUE)
                        self.valid_users_store.set_value(iter, 1, tokens[0])
                    else:
                        self.valid_users_store.set_value(iter, 0, gtk.FALSE)
                        self.valid_users_store.set_value(iter, 1, tokens[0])

    def userToggled(self, data, row):
        iter = self.valid_users_store.get_iter((int(row),))
        val = self.valid_users_store.get_value(iter, 0)
        self.valid_users_store.set_value(iter, 0 , not val)

    def showNewWindow(self):
        self.share_window.set_title(_("Create Samba Share"))
        self.edit_mode = 0
        self.ok_button_handler = self.ok_button.connect("clicked", self.onOkAddButtonClicked)
        self.dir_entry.grab_focus()
        self.populateUserStore()
        self.share_window.show_all()
        self.sharenamechanged = 0

    def showEditWindow(self, iter, parent):
        self.edit_iter = iter
        self.edit_mode = 1
        self.share_window.set_title(_("Edit Samba Share"))
        self.ok_button_handler = self.ok_button.connect("clicked", self.onOkEditButtonClicked)
        self.read_radio.set_active(gtk.TRUE)
        self.sharenamechanged = 0

        userList = []
        invalidUsers = None

        for line in self.samba_file_list:
            if line.type == sambaToken.SambaToken.SAMBA_TOKEN_SECTION_HEADER and string.strip(line.getData()) == parent:
                sharename = string.strip (line.getData(),"\n[]")
                self.sharename_entry.set_text(sharename)
            if line.type == sambaToken.SambaToken.SAMBA_TOKEN_KEYVAL and line.parent == parent:
                if line.keyname == "path":
                    self.dir_entry.set_text(line.keyval)
                elif line.keyname == "comment":
                    if line.keyval != "None":
                        self.description_entry.set_text(line.keyval)
                elif line.keyname == "writeable":
                    if string.lower(line.keyval) == "yes":
                        self.write_radio.set_active(gtk.TRUE)
                    else:
                        self.read_radio.set_active(gtk.TRUE)
                        
                elif string.lower(line.keyname) == "guest ok":
                    if string.lower(line.keyval) == "yes":
                        self.guest_access_radio.set_active(gtk.TRUE)
                    else:
                        self.user_access_radio.set_active(gtk.TRUE)

                if line.keyname == "valid users":
                    if line.keyval:
                        list = string.split(line.keyval, ",")
                        for item in list:
                            userList.append(string.strip(item))

                if line.keyname == "invalid users":
                    if line.keyval and line.keyval == "%S":
                        invalidUsers = line.keyval

        self.sharenamechanged = 0
        self.populateUserStoreOnEdit(userList, invalidUsers)
        self.share_window.show_all()

    def reset(self):
        self.share_notebook.set_current_page(0)
        self.dir_entry.set_text("")
        self.sharename_entry.set_text("")
        self.description_entry.set_text("")
        self.valid_users_store.clear()
        self.read_radio.set_active(gtk.TRUE)
        self.ok_button.disconnect(self.ok_button_handler)
        self.share_window.hide()
        self.sharenamechanged = 0

    def checkDirectoryValidity(self, dir):
        if string.strip(dir) == "":
            dlg = gtk.MessageDialog(self.share_window, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                    (_("You must specify a directory to share.  \n\n"
                                       "Click \"OK\" to continue.")))
            dlg.set_position(gtk.WIN_POS_CENTER)
            dlg.set_modal(gtk.TRUE)
            dlg.set_transient_for(self.share_window)
            dlg.set_icon(mainWindow.iconPixbuf)            
            dlg.run()
            dlg.destroy()
            return 0

        try:
            os.stat(dir)
        except:
            dlg = gtk.MessageDialog(self.share_window, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                    (_("The directory \"%s\" does not exist.  Please specify "
                                       "an existing directory. \n\n"
                                       "Click \"OK\" to continue." % dir)))
            dlg.set_position(gtk.WIN_POS_CENTER)
            dlg.set_modal(gtk.TRUE)
            dlg.set_transient_for(self.share_window)
            dlg.set_icon(mainWindow.iconPixbuf)
            dlg.run()
            dlg.destroy()
            self.share_notebook.set_current_page(0)
            return 0                  

        return 1

    def checkShareNameValidity(self, sharename, path, oldsharename = None):
        msg = None
        buttons = None
        header = "[" + sharename + "]"

        if sharename == "":
            msg = _("Please set a share name.\n\nClick \"OK\" to continue.")
        elif (not oldsharename or oldsharename != sharename) and header in self.samba_data.getHeaders ():
            if header in self.samba_data.getShareHeaders ():
                msg = _("The share name \"%s\" already exists.") % (sharename)
            else:
                msg = _("The share name \"%s\" is reserved.") % (sharename)
            msg += _("\nPlease use a different share name.\n\nClick \"Suggest Share Name\" or \"OK\" to continue.")
            buttons = [(_("_Suggest Share Name"), 2), (gtk.STOCK_OK, 1)]
            self.share_notebook.set_current_page(0)

        if msg:
            dlg = gtk.MessageDialog (self.share_window, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_NONE, msg)
            if buttons:
                for button in buttons:
                    dlg.add_button (button[0], button[1])
            else:
                dlg.add_button (gtk.STOCK_OK, 1)
            dlg.set_position(gtk.WIN_POS_CENTER)
            dlg.set_modal(gtk.TRUE)
            dlg.set_transient_for(self.share_window)
            dlg.set_icon(mainWindow.iconPixbuf)
            result = dlg.run()
            dlg.destroy()
            if result == 2:
                self.sharename_entry.set_text (self.suggestShareName (path, sharename))
                
            return False
        return True

    def suggestShareName (self, path, sharename = None):
        if not sharename or sharename == "":
            if path == "/":
                #Sharing the root is a special case
                sharename = "root directory"

            else:
                #Check to see if the path ends in a "/"  If it does, strip it off
                if path[-1:] == "/":
                    path = path[:-1]

                #If there are any /'s or \'s in the path, split by them
                if '/' in path:
                    tokens = string.split(path, "/")
                    #The last item in the token list is the directory name that we want
                    sharename = tokens[len(tokens)-1]

                #sharename = string.replace (sharename, " ", "_")

	if sharename and sharename != "":
            #If there's already a section header with this name, then start adding numbers to it
            #until it's a unique name
            if ("[" + sharename + "]") in self.samba_data.getHeaders():
                count = 1
                while ("[" + sharename + "]") in self.samba_data.getHeaders():
                    sharename = sharename + "-" + str(count)
                    count = count + 1
        else:
            sharename = ""
 
        return sharename

    def getValidUsers (self):
        all_users = []
        selected_users = []

        user_iter = self.valid_users_store.get_iter_first()
        while user_iter:
            #Crawl through the list and see which users are selected
            all_users.append(self.valid_users_store.get_value(user_iter, 1))
            if self.valid_users_store.get_value(user_iter, 0) == gtk.TRUE:
                selected_users.append(self.valid_users_store.get_value(user_iter, 1))
            user_iter = self.valid_users_store.iter_next(user_iter)

        return (all_users, selected_users)

    def checkValidUsers (self, all_users, selected_users, parent = None):
        if self.user_access_radio.get_active() == gtk.TRUE:
            if selected_users == []:
                #No users are selected.  Make the user choose at least one.
                dlg = gtk.MessageDialog(self.share_window, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                        (_("Please allow access to at least one user.")))
                dlg.set_position(gtk.WIN_POS_CENTER)
                dlg.set_modal(gtk.TRUE)
                dlg.set_transient_for(self.share_window)
                dlg.set_icon(mainWindow.iconPixbuf)
                dlg.run()
                dlg.destroy()
                self.share_notebook.set_current_page(1)
                return False
            elif parent:
                #They have selected at least one user.
                self.ParentClass.changeValue("guest ok", "no", parent)
                                                                                                                                                                                                              
                if all_users == selected_users:
                    #They want to allow all users.  Remove "invalid_users" and "valid users"
                    self.ParentClass.changeValue("valid users", "None", parent)
                    self.ParentClass.changeValue("invalid users", "None", parent)
                else:
                    #They want to allow a subset of all samba users.
                    users = string.join(selected_users, ", ")
                    self.ParentClass.changeValue("valid users", users, parent)
                    self.ParentClass.changeValue("invalid users", "None", parent)

        elif parent and self.guest_access_radio.get_active() == gtk.TRUE:
            self.ParentClass.changeValue("guest ok", "yes", parent)
            self.ParentClass.changeValue("valid users", "None", parent)
            self.ParentClass.changeValue("invalid users", "None", parent)

        return True
 
    #################Event Handlers######################
    def onShareNameEntryChanged(self, *args):
        self.sharenamechanged = 1
        dir_header = "[" + self.sharename_entry.get_text() + "]"

        if self.edit_mode == 1:
            parent = self.ParentClass.share_store.get_value(self.edit_iter, 4)
        else:
            parent = ""
        if parent != dir_header and dir_header in self.samba_data.getShareHeaders():
            count = 1
            while dir_header in self.samba_data.getShareHeaders():
                dir_header = "[" + string.strip(dir_header, "[]") + "-" + str(count) + "]"
                count = count + 1
            self.sharename_entry.set_text (string.strip(dir_header, "[]"))

    def onDirEntryChanged(self, *args):
        if self.sharenamechanged != 1:
            if self.edit_mode == 1:
                parent = self.ParentClass.share_store.get_value(self.edit_iter, 4)
            else:
                parent = ""
            path = string.strip(self.dir_entry.get_text())
            sharename = self.suggestShareName(path)

            self.sharename_entry.set_text(sharename)
            self.sharenamechanged = 0

    def onOkAddButtonClicked(self, *args):
        # Get the path from the widget
        path = self.dir_entry.get_text()
        # Strip off any whitespace
        path = string.strip(path)

        # Get the sharename and strip off any whitespace
        sharename = string.strip(self.sharename_entry.get_text(), "\n[]")

        # Question: Are there other characters that are invalid for the sharename?
        # Is space invalid? Afaik not
        # sharename = string.replace (sharename, " ", "_")

        #Check to see if directory exists
        if not self.checkDirectoryValidity(path):
            return

        #Check to see whether the share name is valid, not duplicate, ...
        if not self.checkShareNameValidity(sharename, path):
            return
        dir_header = "[" + sharename + "]"

        #Check to see if any users are selected.  This will be useful to us later
        (all_users, selected_users) = self.getValidUsers ()
	if not self.checkValidUsers (all_users, selected_users):
            return

        #Ok, things are good now.  Start adding to the share_store
        iter = self.ParentClass.share_store.append()
        self.ParentClass.share_store.set_value(iter, 0, path)
        self.ParentClass.share_store.set_value(iter, 1, sharename)

        #create a blank line token
        if self.samba_data.getList()[-1].getData() != "\n":
            token = self.samba_data.createToken("")
            self.samba_file_list.append(token)
        
        #create token for the section header
        token = self.samba_data.createToken(dir_header)
        self.samba_file_list.append(token)

        #create token for the description if it exists
        description = string.strip(self.description_entry.get_text())

        if description != "":
            while description[-1] == "\\" or description[-1] == " ":
                #If description ends in a backslash, chop it off b/c it confuses Windows
                description = description[:-1]

            token = self.samba_data.createToken("comment = " + description, dir_header)
            self.samba_file_list.append(token)
            self.ParentClass.share_store.set_value(iter, 3, description)

        #create token for the path
        token = self.samba_data.createToken("path = " + path, dir_header)
        self.samba_file_list.append(token)

        #create token for permissions        
        if self.read_radio.get_active() == gtk.TRUE:
            self.ParentClass.share_store.set_value(iter, 2, (_("Read Only")))
        else:
            token = self.samba_data.createToken("read only = no", dir_header)
            self.samba_file_list.append(token)
            self.ParentClass.share_store.set_value(iter, 2, (_("Read / Write")))

        if self.guest_access_radio.get_active() == gtk.TRUE:
            #create token for guest access
            token = self.samba_data.createToken("guest ok = yes", dir_header)
            self.samba_file_list.append(token)
        else:
            if all_users != selected_users:
                #They have selected a subset of all samba users.
                users = string.join(selected_users, ", ")
                token = self.samba_data.createToken("valid users = %s" % users, dir_header)
                self.samba_file_list.append(token)
                
        self.ParentClass.share_store.set_value(iter, 4, dir_header)            
        self.ParentClass.properties_button.set_sensitive(gtk.FALSE)
        self.ParentClass.delete_button.set_sensitive(gtk.FALSE)
        self.ParentClass.share_view.get_selection().unselect_all()
        self.reset()

        #Let's go ahead and restart the service.
        self.samba_data.writeFile()
        self.samba_backend.restartSamba()

    def onOkEditButtonClicked(self, *args):
        parent = self.ParentClass.share_store.get_value(self.edit_iter, 4)
        oldsharename = self.ParentClass.share_store.get_value(self.edit_iter, 1)

        dir = self.dir_entry.get_text()
        dir = string.strip(dir)

        # Get the sharename and strip off any whitespace
        sharename = string.strip(self.sharename_entry.get_text(), "\n[]")

        #Check to see if directory exists
        if not self.checkDirectoryValidity(dir):
            return

        #Check to see whether the share name is valid, not duplicate, ...
        if not self.checkShareNameValidity(sharename, dir, oldsharename):
            return
        dir_header = "[" + sharename + "]"

        #Check to see if any users are selected.  This will be useful to us later
        (all_users, selected_users) = self.getValidUsers ()
        if not self.checkValidUsers (all_users, selected_users, parent):
            return

        # parent contains [the old sharename] of course
        # If the new name differs from the old one, we will need to rename
        oldsharename = string.strip (parent,"\n[]")
        if sharename != oldsharename:
            self.ParentClass.share_store.set_value(self.edit_iter, 1, sharename)
            self.ParentClass.share_store.set_value(self.edit_iter, 4, "[" + sharename + "]")
            self.ParentClass.changeSectionHeader (oldsharename, sharename)
            self.samba_file_list = self.ParentClass.samba_data.getList()
            for line in self.samba_file_list:
                if line.type == sambaToken.SambaToken.SAMBA_TOKEN_KEYVAL and line.parent == "[" + oldsharename + "]":
                    line.parent = "[" + sharename + "]"

        self.ParentClass.share_store.set_value(self.edit_iter, 0, dir)
        self.ParentClass.changeValue("path", dir, parent)

        if self.read_radio.get_active() == gtk.TRUE:
            self.ParentClass.share_store.set_value(self.edit_iter, 2, (_("Read Only")))
            self.ParentClass.changeValue("writeable", "no", parent)
        else:
            self.ParentClass.share_store.set_value(self.edit_iter, 2, (_("Read / Write")))
            self.ParentClass.changeValue("writeable", "yes", parent)

        description = string.strip(self.description_entry.get_text())

        if description != "":
            while description[-1] == "\\" or description[-1] == " ":
                #If description ends in a backslash, chop it off b/c it confuses Windows
                description = description[:-1]

        if description == "":
            self.ParentClass.share_store.set_value(self.edit_iter, 3, description)
            self.ParentClass.changeValue("comment", "None", parent)
        else:
            self.ParentClass.share_store.set_value(self.edit_iter, 3, description)
            self.ParentClass.changeValue("comment", description, parent)

        self.ParentClass.properties_button.set_sensitive(gtk.FALSE)
        self.ParentClass.delete_button.set_sensitive(gtk.FALSE)
        self.ParentClass.share_view.get_selection().unselect_all()
        self.reset()

        #Let's go ahead and restart the service.
        self.samba_data.writeFile()
        self.samba_backend.restartSamba()

    def onCancelButtonClicked(self, *args):
        self.reset()
        return gtk.TRUE

    def onBrowseButtonClicked(self, *args):
        fb = gtk.FileSelection(_("Select Directory"))
        fb.set_position(gtk.WIN_POS_CENTER)
        fb.set_icon(mainWindow.iconPixbuf)
        fb.show_all()
        result = fb.run()        
        fb.hide()

        if result == gtk.RESPONSE_OK:
            self.dir_entry.set_text(fb.get_filename())

    def userRadioToggled(self, *args):
        self.valid_users_treeview.set_sensitive(self.user_access_radio.get_active())
